@@ -6,70 +6,156 @@
 */
 
 #include "windoweffects.h"
-#include "waylandintegration.h"
 
 #include <QDebug>
 #include <QExposeEvent>
 #include <QGuiApplication>
 #include <QWidget>
 
-#include <KWayland/Client/blur.h>
-#include <KWayland/Client/compositor.h>
-#include <KWayland/Client/connection_thread.h>
-#include <KWayland/Client/contrast.h>
-#include <KWayland/Client/plasmashell.h>
-#include <KWayland/Client/plasmawindowmanagement.h>
-#include <KWayland/Client/region.h>
-#include <KWayland/Client/registry.h>
-#include <KWayland/Client/slide.h>
-#include <KWayland/Client/surface.h>
 #include <private/qwaylandwindow_p.h>
 
+#include <QWaylandClientExtensionTemplate>
+#include <qwaylandclientextension.h>
+
+#include "qwayland-blur.h"
+#include "qwayland-contrast.h"
+#include "qwayland-slide.h"
+
+#include "surfacehelper.h"
+
+static wl_region *createRegion(const QRegion &region)
+{
+    QPlatformNativeInterface *native = qGuiApp->platformNativeInterface();
+    if (!native) {
+        return nullptr;
+    }
+    auto compositor = reinterpret_cast<wl_compositor *>(native->nativeResourceForIntegration(QByteArrayLiteral("compositor")));
+    if (!compositor) {
+        return nullptr;
+    }
+    auto wl_region = wl_compositor_create_region(compositor);
+    for (const auto &rect : region) {
+        wl_region_add(wl_region, rect.x(), rect.y(), rect.width(), rect.height());
+    }
+    return wl_region;
+}
+
+class BlurManager : public QWaylandClientExtensionTemplate<BlurManager>, public QtWayland::org_kde_kwin_blur_manager
+{
+public:
+    BlurManager()
+        : QWaylandClientExtensionTemplate<BlurManager>(1)
+    {
+    }
+};
+
+class Blur : public QObject, public QtWayland::org_kde_kwin_blur
+{
+public:
+    Blur(struct ::org_kde_kwin_blur *object, QObject *parent)
+        : QObject(parent)
+        , QtWayland::org_kde_kwin_blur(object)
+    {
+    }
+
+    ~Blur() override
+    {
+        release();
+    }
+};
+
+class ContrastManager : public QWaylandClientExtensionTemplate<ContrastManager>, public QtWayland::org_kde_kwin_contrast_manager
+{
+public:
+    ContrastManager()
+        : QWaylandClientExtensionTemplate<ContrastManager>(2)
+    {
+    }
+};
+
+class Contrast : public QObject, public QtWayland::org_kde_kwin_contrast
+{
+public:
+    Contrast(struct ::org_kde_kwin_contrast *object, QObject *parent)
+        : QObject(parent)
+        , QtWayland::org_kde_kwin_contrast(object)
+    {
+    }
+
+    ~Contrast() override
+    {
+        release();
+    }
+};
+
+class SlideManager : public QWaylandClientExtensionTemplate<SlideManager>, public QtWayland::org_kde_kwin_slide_manager
+{
+public:
+    SlideManager()
+        : QWaylandClientExtensionTemplate<SlideManager>(1)
+    {
+    }
+};
+
+class Slide : public QObject, public QtWayland::org_kde_kwin_slide
+{
+public:
+    Slide(struct ::org_kde_kwin_slide *object, QObject *parent)
+        : QObject(parent)
+        , QtWayland::org_kde_kwin_slide(object)
+    {
+    }
+
+    ~Slide() override
+    {
+        release();
+    }
+};
+
 WindowEffects::WindowEffects()
     : QObject()
-    ,  KWindowEffectsPrivateV2()
+    , KWindowEffectsPrivateV2()
 {
-    auto registry = WaylandIntegration::self()->registry();
+    m_blurManager = new BlurManager();
+    m_contrastManager = new ContrastManager();
+    m_slideManager = new SlideManager();
 
     // The KWindowEffects API doesn't provide any signals to notify that the particular
     // effect has become unavailable. So we re-install effects when the corresponding globals
     // are added.
-    connect(registry, &KWayland::Client::Registry::blurAnnounced, this, [this]() {
+
+    connect(m_blurManager, &BlurManager::activeChanged, this, [this] {
         for (auto it = m_blurRegions.constBegin(); it != m_blurRegions.constEnd(); ++it) {
-            installBlur(it.key(), true, *it);
-        }
-    });
-    connect(registry, &KWayland::Client::Registry::blurRemoved, this, [this]() {
-        for (auto it = m_blurRegions.constBegin(); it != m_blurRegions.constEnd(); ++it) {
-            installBlur(it.key(), false, *it);
+            installBlur(it.key(), m_blurManager->isActive(), *it);
         }
     });
 
-    connect(registry, &KWayland::Client::Registry::contrastAnnounced, this, [this]() {
+    connect(m_contrastManager, &ContrastManager::activeChanged, this, [this] {
         for (auto it = m_backgroundConstrastRegions.constBegin(); it != m_backgroundConstrastRegions.constEnd(); ++it) {
-            installContrast(it.key(), true, it->contrast, it->intensity, it->saturation, it->region);
-        }
-    });
-    connect(registry, &KWayland::Client::Registry::contrastRemoved, this, [this]() {
-        for (auto it = m_backgroundConstrastRegions.constBegin(); it != m_backgroundConstrastRegions.constEnd(); ++it) {
-            installContrast(it.key(), false);
+            if (m_contrastManager->isActive()) {
+                installContrast(it.key(), true, it->contrast, it->intensity, it->saturation, it->region);
+            } else {
+                installContrast(it.key(), false);
+            }
         }
     });
 
-    connect(registry, &KWayland::Client::Registry::slideAnnounced, this, [this]() {
+    connect(m_slideManager, &SlideManager::activeChanged, this, [this] {
         for (auto it = m_slideMap.constBegin(); it != m_slideMap.constEnd(); ++it) {
-            installSlide(it.key(), it->location, it->offset);
-        }
-    });
-    connect(registry, &KWayland::Client::Registry::slideRemoved, this, [this]() {
-        for (auto it = m_slideMap.constBegin(); it != m_slideMap.constEnd(); ++it) {
-            installSlide(it.key(), KWindowEffects::SlideFromLocation::NoEdge, 0);
+            if (m_slideManager->isActive()) {
+                installSlide(it.key(), it->location, it->offset);
+            } else {
+                installSlide(it.key(), KWindowEffects::SlideFromLocation::NoEdge, 0);
+            }
         }
     });
 }
 
 WindowEffects::~WindowEffects()
 {
+    delete m_blurManager;
+    delete m_contrastManager;
+    delete m_slideManager;
 }
 
 QWindow *WindowEffects::windowForId(WId wid)
@@ -132,12 +218,12 @@ void replaceValue(MapType &map, typename MapType::key_type key, typename MapType
     }
 }
 
-void WindowEffects::resetBlur(QWindow *window, KWayland::Client::Blur *blur)
+void WindowEffects::resetBlur(QWindow *window, Blur *blur)
 {
     replaceValue(m_blurs, window, blur);
 }
 
-void WindowEffects::resetContrast(QWindow *window, KWayland::Client::Contrast *contrast)
+void WindowEffects::resetContrast(QWindow *window, Contrast *contrast)
 {
     replaceValue(m_contrasts, window, contrast);
 }
@@ -168,6 +254,12 @@ bool WindowEffects::eventFilter(QObject *watched, QEvent *event)
                 installContrast(window, true, it->contrast, it->intensity, it->saturation, it->region);
             }
         }
+        {
+            auto it = m_slideMap.constFind(window);
+            if (it != m_slideMap.constEnd()) {
+                installSlide(window, it->location, it->offset);
+            }
+        }
     }
     return false;
 }
@@ -176,11 +268,11 @@ bool WindowEffects::isEffectAvailable(KWindowEffects::Effect effect)
 {
     switch (effect) {
     case KWindowEffects::BackgroundContrast:
-        return WaylandIntegration::self()->waylandContrastManager() != nullptr;
+        return m_contrastManager->isActive();
     case KWindowEffects::BlurBehind:
-        return WaylandIntegration::self()->waylandBlurManager() != nullptr;
+        return m_blurManager->isActive();
     case KWindowEffects::Slide:
-        return WaylandIntegration::self()->waylandSlideManager() != nullptr;
+        return m_slideManager->isActive();
     default:
         return false;
     }
@@ -208,74 +300,64 @@ void WindowEffects::slideWindow(WId id, KWindowEffects::SlideFromLocation locati
 
 void WindowEffects::installSlide(QWindow *window, KWindowEffects::SlideFromLocation location, int offset)
 {
-    if (!WaylandIntegration::self()->waylandSlideManager()) {
+    if (!m_slideManager->isActive()) {
         return;
     }
-    KWayland::Client::Surface *surface = KWayland::Client::Surface::fromWindow(window);
+    wl_surface *surface = surfaceForWindow(window);
     if (surface) {
         if (location != KWindowEffects::SlideFromLocation::NoEdge) {
-            auto slide = WaylandIntegration::self()->waylandSlideManager()->createSlide(surface, surface);
+            auto slide = new Slide(m_slideManager->create(surface), window);
 
-            KWayland::Client::Slide::Location convertedLoc;
+            Slide::location convertedLoc;
             switch (location) {
             case KWindowEffects::SlideFromLocation::TopEdge:
-                convertedLoc = KWayland::Client::Slide::Location::Top;
+                convertedLoc = Slide::location::location_top;
                 break;
             case KWindowEffects::SlideFromLocation::LeftEdge:
-                convertedLoc = KWayland::Client::Slide::Location::Left;
+                convertedLoc = Slide::location::location_left;
                 break;
             case KWindowEffects::SlideFromLocation::RightEdge:
-                convertedLoc = KWayland::Client::Slide::Location::Right;
+                convertedLoc = Slide::location::location_right;
                 break;
             case KWindowEffects::SlideFromLocation::BottomEdge:
             default:
-                convertedLoc = KWayland::Client::Slide::Location::Bottom;
+                convertedLoc = Slide::location::location_bottom;
                 break;
             }
 
-            slide->setLocation(convertedLoc);
-            slide->setOffset(offset);
+            slide->set_location(convertedLoc);
+            slide->set_offset(offset);
             slide->commit();
         } else {
-            WaylandIntegration::self()->waylandSlideManager()->removeSlide(surface);
+            m_slideManager->unset(surface);
         }
-
-        WaylandIntegration::self()->waylandConnection()->flush();
     }
 }
 
-#if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 81)
 QList<QSize> WindowEffects::windowSizes(const QList<WId> &ids)
 {
     Q_UNUSED(ids)
     QList<QSize> sizes;
     return sizes;
 }
-#endif
 
-#if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 82)
 void WindowEffects::presentWindows(WId controller, const QList<WId> &ids)
 {
     Q_UNUSED(controller)
     Q_UNUSED(ids)
 }
-#endif
 
-#if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 82)
 void WindowEffects::presentWindows(WId controller, int desktop)
 {
     Q_UNUSED(controller)
     Q_UNUSED(desktop)
 }
-#endif
 
-#if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 82)
 void WindowEffects::highlightWindows(WId controller, const QList<WId> &ids)
 {
     Q_UNUSED(controller)
     Q_UNUSED(ids)
 }
-#endif
 
 void WindowEffects::enableBlurBehind(WId winId, bool enable, const QRegion &region)
 {
@@ -297,28 +379,32 @@ void WindowEffects::enableBlurBehind(WId winId, bool enable, const QRegion &regi
 
 void WindowEffects::installBlur(QWindow *window, bool enable, const QRegion &region)
 {
-    if (!WaylandIntegration::self()->waylandBlurManager()) {
+    if (!m_blurManager->isActive()) {
         return;
     }
-    KWayland::Client::Surface *surface = KWayland::Client::Surface::fromWindow(window);
+
+    wl_surface *surface = surfaceForWindow(window);
 
     if (surface) {
         if (enable) {
-            auto blur = WaylandIntegration::self()->waylandBlurManager()->createBlur(surface, surface);
-            std::unique_ptr<KWayland::Client::Region> wlRegion(WaylandIntegration::self()->waylandCompositor()->createRegion(region, nullptr));
-            blur->setRegion(wlRegion.get());
+            auto wl_region = createRegion(region);
+            if (!wl_region) {
+                return;
+            }
+            auto blur = new Blur(m_blurManager->create(surface), window);
+            blur->set_region(wl_region);
             blur->commit();
+            wl_region_destroy(wl_region);
             resetBlur(window, blur);
         } else {
             resetBlur(window);
-            WaylandIntegration::self()->waylandBlurManager()->removeBlur(surface);
+            m_blurManager->unset(surface);
         }
-
-        WaylandIntegration::self()->waylandConnection()->flush();
     }
 }
 
 void WindowEffects::enableBackgroundContrast(WId winId, bool enable, qreal contrast, qreal intensity, qreal saturation, const QRegion &region)
+
 {
     auto window = windowForId(winId);
     if (!window) {
@@ -341,58 +427,61 @@ void WindowEffects::enableBackgroundContrast(WId winId, bool enable, qreal contr
 
 void WindowEffects::installContrast(QWindow *window, bool enable, qreal contrast, qreal intensity, qreal saturation, const QRegion &region)
 {
-    if (!WaylandIntegration::self()->waylandContrastManager()) {
+    if (!m_contrastManager->isActive()) {
         return;
     }
-    KWayland::Client::Surface *surface = KWayland::Client::Surface::fromWindow(window);
+
+    wl_surface *surface = surfaceForWindow(window);
     if (surface) {
         if (enable) {
-            auto backgroundContrast = WaylandIntegration::self()->waylandContrastManager()->createContrast(surface, surface);
-            std::unique_ptr<KWayland::Client::Region> wlRegion(WaylandIntegration::self()->waylandCompositor()->createRegion(region, nullptr));
-            backgroundContrast->setRegion(wlRegion.get());
-            backgroundContrast->setContrast(contrast);
-            backgroundContrast->setIntensity(intensity);
-            backgroundContrast->setSaturation(saturation);
+            auto wl_region = createRegion(region);
+            if (!wl_region) {
+                return;
+            }
+            auto backgroundContrast = new Contrast(m_contrastManager->create(surface), window);
+            backgroundContrast->set_region(wl_region);
+            backgroundContrast->set_contrast(wl_fixed_from_double(contrast));
+            backgroundContrast->set_intensity(wl_fixed_from_double(intensity));
+            backgroundContrast->set_saturation(wl_fixed_from_double(saturation));
             backgroundContrast->commit();
+            wl_region_destroy(wl_region);
             resetContrast(window, backgroundContrast);
         } else {
             resetContrast(window);
-            WaylandIntegration::self()->waylandContrastManager()->removeContrast(surface);
+            m_contrastManager->unset(surface);
         }
-
-        WaylandIntegration::self()->waylandConnection()->flush();
     }
 }
 
 void WindowEffects::setBackgroundFrost(QWindow *window, QColor color, const QRegion &region)
 {
-    if (!WaylandIntegration::self()->waylandContrastManager()) {
+    if (!m_contrastManager->isActive()) {
         return;
     }
 
-    KWayland::Client::Surface *surface = KWayland::Client::Surface::fromWindow(window);
+    wl_surface *surface = surfaceForWindow(window);
     if (!surface) {
         return;
     }
     if (!color.isValid()) {
         resetContrast(window);
-        WaylandIntegration::self()->waylandContrastManager()->removeContrast(surface);
+        m_contrastManager->unset(surface);
         return;
     }
 
-    auto backgroundContrast = WaylandIntegration::self()->waylandContrastManager()->createContrast(surface, surface);
-    std::unique_ptr<KWayland::Client::Region> wlRegion(WaylandIntegration::self()->waylandCompositor()->createRegion(region, nullptr));
-    backgroundContrast->setRegion(wlRegion.get());
-    backgroundContrast->setFrost(color);
+    auto wl_region = createRegion(region);
+    if (!wl_region) {
+        return;
+    }
+    auto backgroundContrast = new Contrast(m_contrastManager->create(surface), window);
+    backgroundContrast->set_region(wl_region);
+    backgroundContrast->set_frost(color.red(), color.green(), color.blue(), color.alpha());
     backgroundContrast->commit();
+    wl_region_destroy(wl_region);
     resetContrast(window, backgroundContrast);
-
-    WaylandIntegration::self()->waylandConnection()->flush();
 }
 
-#if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 67)
 void WindowEffects::markAsDashboard(WId window)
 {
     Q_UNUSED(window)
 }
-#endif
